/**
 * Casebuilder widget that emits events upon tab-selection changes.
 * 
 * Usefull to manually refresh/trigger things, or for lazy tab loading.
 * 
 * Licensed under MIT (license.txt).
 * 
 * @author ivojonker.nl
 */
define([
	"dojo/_base/declare",
	"dojo/json",
	"icm/base/BasePageWidget",
	"icm/base/_BaseWidget", 
	"dijit/form/Button",
	"dijit/layout/TabContainer",
	"dojo/_base/lang",
	"dojo/aspect"
], function(declare, json, BasePageWidget, _BaseWidget, Button, TabContainer,lang, aspect){
	
	var CONST_WIRE_CREATED="TabContainerEventAdapter.created";
	var CONST_WIRE_OPENED="TabContainerEventAdapter.enclosingtab.opened";
	var CONST_WIRE_CLOSED="TabContainerEventAdapter.enclosingtab.closed";
	var CONST_WIRE_REFRESHED="TabContainerEventAdapter.enclosingtab.refreshed";
	
	
	return declare("nl.dj8.icmwidgets.widget.TabContainerEventAdapter", [_BaseWidget, BasePageWidget], {
		templateString: "<div style='display:none'></div>",

		postCreate: function(){
			this._attachToTabContainerEvents();
			this._emitWiring(CONST_WIRE_CREATED, true);
		},
		
		handleICM_setStoredPayload :function (payload){
			this.storedPayload=payload;
		},

		/**
		 * Attaches both to selectCildWidget as selectchild to be able to differentiate between swaps and refreshes.
		 */
		_attachToTabContainerEvents : function(){
			this.tabContainerContext = this._findEnclosingTabContainerContext();
			if (this.tabContainerContext){
				var didSwapTab=false;
				
				this.tabContainerContext.tabContainer.watch("selectedChildWidget", lang.hitch(this,function(name,closed,opened){
					didSwapTab=true;
					if (opened===this.tabContainerContext.page){
						this._emitWiring(CONST_WIRE_OPENED);
					}else if (closed===this.tabContainerContext.page){
						this._emitWiring(CONST_WIRE_CLOSED);
					}
				}));
				
				aspect.after(this.tabContainerContext.tabContainer,"selectChild",lang.hitch(this,function(){
					if (!didSwapTab && (this.tabContainerContext.tabContainer.selectedChildWidget===this.tabContainerContext.page)){
						this._emitWiring(CONST_WIRE_REFRESHED);
					}
					didSwapTab=false;
				}));
			}else{
				console.error("Pagebuilder / configuration error: TabContainerEventAdapter was located outside of a tabcontainer context and could not attach.");
				console.error(this);
			}
		},
		
		_constructPayload : function(wire,tabContainerContext){
			try{
				return payload=eval("(function(){"+this.widgetProperties.payload+"}).apply(this)");
			}catch(e){
				console.error("Error while trying to construct a payload following a tab-event.");
				console.error("Error:" + e);
				console.error("For script: "+script);
				return e;
			}
		},
		
		/**
		 * Emits, asynchronously to allow other widget to instantiate (required for the creation wireevent);
		 */
		_emitWiring : function(wire){
			setTimeout(lang.hitch(this, function(){
				this.onPublishEvent(wire, this._constructPayload(wire,this.tabContainerContext));
			}),1);
		},
		
		/**
		 * Recursively cralws up to the root node to find the nearest implementation of a tabcontainer, also keeping reference to the page that corresponds to this widget.
		 */
		_findEnclosingTabContainerContext : function(){
			var currPotentialPage = this;
			while (currPotentialPage.getParent()){
				if (currPotentialPage.getParent().isInstanceOf(TabContainer))
					return { page : currPotentialPage, tabContainer : currPotentialPage.getParent()};
				else
					currPotentialPage = currPotentialPage.getParent();	
			}
		}
		
	});
});
